package com.ibm.mqe.model;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import com.ibm.uia.renderingInterface.*;
import java.awt.Color;
import java.util.*;
import com.ibm.mqe.*;
import com.ibm.uia.metamodel.*;
/**
 * This class represents those proxies that can be create and
 * deleted. (Everything but QueueManagers?)
 * Creation date: (05/07/2000 07:19:59)
 * @author: Phill van Leersum
 */
public abstract class CreatableObject extends MQeObject {
	private static final String STATE_TOBECREATED = "To be created";
	private static final String STATE_CREATING = "Creating";
	private static final String STATE_CREATED = "Created";
	private static final String STATE_TOBEDELETED = "To be deleted";
	private static final String STATE_DELETED = "Deleted";
	private static final String STATE_DELETING = "Deleting";
	private String state;
/* All committing of deletes must be done before
 * committing creates, incase we are moving an object.
 * If we are moving an object and try to create it in
 * the new place before deleting it in the old, we
 * may get a name clash
 */
private boolean _commitDelete(boolean recurse) {
	boolean changed = false;
	if (isToBeDeleted()) {
		changed = performDelete();
	} else {
		if (recurse) {
			Enumeration enum = getAllChildren();
			while (enum.hasMoreElements()) {
				CreatableObject mqeo = (CreatableObject) enum.nextElement();
				if (mqeo._commitDelete(recurse)) { 
					changed = true;
				}
			}
		}
	}
	return changed;
}
/**
 * Read the receivers data from the output data field
 */
protected final boolean _handleCreateReply(MQeAdminMsg msg) {
	boolean reply = false;
	if (!isCreating()) {
		_error("create reply when not creating");
	} else {
		if (_isSuccess(msg)) {
			setStateCreated();
			refresh();
		} else {
			setStateToBeCreated();
			_warning("create failed: " + _getReason(msg));
		}
		reply = true;
	}
	return reply;
}
protected final boolean _handleDeleteReply(MQeAdminMsg msg) {
	boolean changed = false;
	if (!isDeleting()) {
		_error("delete reply when not deleting");
	} else {
		if (_isSuccess(msg)) {
			getParent().removeChild(this);
			setStateDeleted();
		} else {
			_warning("delete failed: " +_getReason(msg)); 
			setStateToBeDeleted();
		}
		changed = true;
	}
	return changed;
}
/**
 * _readRefreshData method comment.
 */
protected boolean _readRefreshData(MQeFields data, Vector found) {
	if (isCreating() || isToBeCreated()) {
		setStateCreated();
		return true;
	}
	return false;
}
/**
 * If the receiver is connected, then attempt to commit all outstanding
 * changes (creates/deletes/updates).
 * Overide here to handle the deletes BEFORE the creates.  This is important
 * when a name clash might occur (eg cut and paste).
 */
public boolean commit(boolean recurse) {
	boolean changed = false;
	if (isConnected()) {  // only commit if connected
		changed = _commitDelete(recurse);
		if (isToBeCreated()) {
			if (performCreate()) {
				changed = true;
			}
		}
	}
	if (super.commit(recurse)) {
		changed = true;
	}
	return changed;
}
public final boolean cut() {
	copy();
	delete();
	commit(false);
	return true;
}
/**
 * The user has requested a deletion
 */
public boolean delete() {
	if (isToBeCreated() || isCreating()) {
		getParent().removeChild(this);
	} else {
		setStateToBeDeleted();
		commit(false);
	}
	return true;
}
public final Color getDisplayColour() {
	if (isCategory()) {
		return Color.black;
	} else if (isToBeCreated()) {
		return Color.blue;
	} else if (isCreating()) {
		return Color.cyan;
	} else if (isToBeDeleted()) {
		return Color.red;
	} else if (isDeleting()) {
		return Color.magenta;
	}
	return Color.black;
}
public String getHintText() {
	if (isCategory()) {
		return getName();
	} else {
		return getType() + ": " + getName() + " (" + state + ")";
	}
}
protected final String getState() {
	return state;
}
/**
 * This is the method called to handle all reply messages, whether delivered
 * asynchronously (through the callback method 'acceptMessage') or synchronously
 * as a returned value from a stub operation.
 */
protected boolean handleMessage(MQeAdminMsg msg) {
	boolean changed = false;
	if (null != msg) {
		switch (_getAction(msg)) {
			case MQeAdminMsg.Action_Delete :
				changed = _handleDeleteReply(msg);
				break;
			case MQeAdminMsg.Action_Create :
				changed = _handleCreateReply(msg);
				break;
			default :
				changed = super.handleMessage(msg);
				break;
		}
	}
	return changed;
}
protected void initialise(String name, MetaObject metaObject) {
	super.initialise(name, metaObject);
	setStateToBeCreated();
}
public final boolean isCreated() {
	return STATE_CREATED == state;  // can use identity, since we are using const strings
}
public final boolean isCreating() {
	return STATE_CREATING == state;  // can use identity, since we are using const strings
}
public final boolean isDeleted() {
	return STATE_DELETED == state;  // can use identity, since we are using const strings
}
public final boolean isDeleting() {
	return STATE_DELETING == state;  // can use identity, since we are using const strings
}
public final boolean isToBeCreated() {
	if (isCategory()) {state = STATE_CREATED;}
	return STATE_TOBECREATED == state;  // can use identity, since we are using const strings
}
public final boolean isToBeDeleted() {
	return STATE_TOBEDELETED == state;  // can use identity, since we are using const strings
}
protected boolean performCreate() {
	setStateCreating();
	return false;
}
protected boolean performDelete() {
	setStateDeleting();
	return false;
}
public final void revert() {
	super.revert();
	if (isToBeDeleted()) {
		setStateCreated();
	} else if (isToBeCreated()) {
		getParent().removeChild(this);
	}
}
protected final void setState(String state) {
	this.state = state;
}
public final void setStateCreated() {
	state = STATE_CREATED;
}
protected final void setStateCreating() {
	_setAttributeEditable(OBJECT_NAME, false);
	state = STATE_CREATING;
}
protected final void setStateDeleted() {
	state = STATE_DELETED;
}
protected final void setStateDeleting() {
	state = STATE_DELETING;
}
public final void setStateToBeCreated() {
	_setAttributeEditable(OBJECT_NAME, true);
	state = STATE_TOBECREATED;
}
public final void setStateToBeDeleted() {
	state = STATE_TOBEDELETED;
}
}
